<?php
/**
 * @package    miniOrange
 * @subpackage Plugins
 * @license    GNU/GPLv3
 * @copyright  Copyright 2015 miniOrange. All Rights Reserved.
 *
 *
 * This file is part of miniOrange Joomla SAML IDP plugin.
 *
 * miniOrange Joomla SAML IDP plugin is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * miniOrange Joomla IDP plugin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with miniOrange SAML plugin.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @package     Joomla.Administrator
 * @subpackage  com_joomlaidp
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access to this file
defined('_JEXEC') or die('Restricted access');
 
/**
 * AccountSetup Controller
 *
 * @package     Joomla.Administrator
 * @subpackage  com_joomlaidp
 * @since       0.0.9
 */
class JoomlaIdpControllerAccountSetup extends JControllerForm
{
	function __construct()
	{
		$this->view_list = 'accountsetup';
		parent::__construct();
	}


	function saveServiceProvider(){

		$post = JFactory::getApplication()->input->post->getArray();
        if(!isset($post['sp_name']) && !isset($post['sp_entityid']) && !isset($post['acs_url'])){
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp');
            return;
        }
		$isDelete = isset($post['mo_saml_delete']) ? $post['mo_saml_delete'] : '';

		if ($isDelete == "Delete SP Configuration")
        {
            $data = new stdClass();
            $data->id = 1;
            $data->sp_name = '';
            $data->sp_entityid = '';
            $data->acs_url = '';
            $data->nameid_format = '';
            $data->nameid_attribute = '';
            $data->default_relay_state = '';
            $data->assertion_signed = 0;
            $data->enabled = 0;

            $db = JFactory::getDBO();
            $db->updateObject( '#__miniorangesamlidp', $data, 'id', true);

            $message = JText::_('COM_JOOMLAIDP_MSG_5');
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', $message );
        }
		else{
            $spName = isset($post['sp_name']) ? $post['sp_name'] : '';
            $issuer = isset($post['sp_entityid']) ? $post['sp_entityid'] : '';
            $acsUrl = isset($post['acs_url']) ? $post['acs_url'] : '';
            $nameIdFormat = isset($post['nameid_format']) ? $post['nameid_format'] : '';
            $defaultRelayState = isset($post['default_relay_state']) ? $post['default_relay_state'] : '';
            $assertionSigned = isset($post['assertion_signed']) ? isset($post['assertion_signed']) : 0;
            if(empty($spName) || empty($issuer) || empty($acsUrl) || empty($nameIdFormat)){
                $message = JText::_('COM_JOOMLAIDP_MSG_6');
                $this->setRedirect('index.php?option=com_joomlaidp&view=samlidpsettings',  $message,'error');
                return FALSE;
            }

            $spName = strtolower(trim($spName));
            $issuer = trim($issuer);
            $acsUrl = trim($acsUrl);

            $data = new stdClass();
            $data->id = 1;
            $data->sp_name = $spName;
            $data->sp_entityid = $issuer;
            $data->acs_url = $acsUrl;
            $data->nameid_format = $nameIdFormat;
            $data->nameid_attribute = empty($post['nameid_attribute']) ? 'emailAddress' : $post['nameid_attribute'];
            $data->default_relay_state = $defaultRelayState;
            $data->assertion_signed = $assertionSigned;
            $data->enabled = TRUE;

            $db = JFactory::getDBO();
            $db->updateObject( '#__miniorangesamlidp', $data, 'id', true);

            IDP_Utilities::isValidCheck($spName, $acsUrl,'Save Details','');
            $message = JText::_('COM_JOOMLAIDP_MSG_7') .' (' . $spName . ') '.JText::_('COM_JOOMLAIDP_MSG_8').' <a href="index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp&test-config=true" class="mo_boot_btn mo_boot_btn-saml">'.JText::_('COM_JOOMLAIDP_TEST_CONFIG').'</a>'.JText::_('COM_JOOMLAIDP_MSG_9');
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', $message );
        }
	}

	

	function handleUploadMetadata(){

		require_once JPATH_COMPONENT.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'MetadataReader.php';
		$post = JFactory::getApplication()->input->post->getArray();
        if(count($post) == 0){
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp');
            return;
        }
        $file = JFactory::getApplication()->input->files->getArray();

        if ( !isset($post['sp_upload_name']) || empty($post['sp_upload_name'])) {
        	$this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', JText::_('COM_JOOMLAIDP_MSG_10'),'error');
        	return;
        }

        $sp_name = $post['sp_upload_name'];

        if (isset($file['metadata_file']) || isset($post['metadata_url'])) {
            if(!empty($file['metadata_file']['tmp_name'])) {
                $file = @file_get_contents( $file['metadata_file']['tmp_name']);
            }
            else {
                $url = filter_var($post['metadata_url'],FILTER_SANITIZE_URL);
                $arrContextOptions=array(
                    "ssl"=>array(
                        "verify_peer"=>false,
                        "verify_peer_name"=>false,
                    ),
                );
                if(empty($url)) {
                    $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', JText::_('COM_JOOMLAIDP_MSG_11'),'error');
                    return;
                }
                else {
                    $file = file_get_contents($url, false, stream_context_create($arrContextOptions));
                }
            }
            $this->uploadMetadata($file, $sp_name);
        }
	}

	function uploadMetadata($file, $sp_name){

		$post = JFactory::getApplication()->input->post->getArray();
        if(count($post) == 0){
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp');
            return;
        }
		$document = new DOMDocument();
        $document->loadXML( $file );
        restore_error_handler();
        $first_child = $document->firstChild;

        if( !empty( $first_child ) ) {
            $metadata = new MetadataReader($document);
            $service_providers = $metadata->getServiceProviders();
            if( empty( $service_providers ) ) {
                $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp',  JText::_('COM_JOOMLAIDP_MSG_12'),'error');
                return;
            }
            foreach( $service_providers as $key => $sp ) {
                $issuer = $sp->getEntityID();
                $acs_url = $sp->getAcsURL();
                $is_assertion_signed = $sp->getAssertionsSigned() == 'true' ? TRUE : FALSE;
            }
            $data = new stdClass();
			$data->id = 1;
			$data->sp_name = $sp_name;
			$data->sp_entityid = $issuer;
			$data->acs_url = $acs_url;
			$data->nameid_format = empty($post['nameid_format']) ? 'urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified' : $post['nameid_format'];
        
			$data->nameid_attribute = empty($post['nameid_attribute']) ? 'emailAddress' : $post['nameid_attribute'];
			$data->assertion_signed = $is_assertion_signed;
			$data->enabled = TRUE;
		
		    $db = JFactory::getDBO();
		    $db->updateObject( '#__miniorangesamlidp', $data, 'id', true);
            IDP_Utilities::isValidCheck($sp_name, $acs_url,'Save Details','');
		    $message = JText::_('COM_JOOMLAIDP_MSG_7') .' (' . $sp_name . ') '.JText::_('COM_JOOMLAIDP_MSG_8').' <a href="index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp&test-config=true" class="mo_boot_btn mo_boot_btn-saml">'.JText::_('COM_JOOMLAIDP_TEST_CONFIG').'</a>'.JText::_('COM_JOOMLAIDP_MSG_9');
		    $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', $message );
            return;
        }
        else {
        	$this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp', JText::_('COM_JOOMLAIDP_MSG_13'),'error');
        	return;
        }
	}

	function updateIdpEntityId()
	{
		$post = JFactory::getApplication()->input->post->getArray();
        if(count($post) == 0){
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=idp');
            return;
        }
		$newIdp = $post['mo_saml_idp_entity_id'];

        $nameOfDatabase = '#__miniorange_saml_idp_customer';
        $updateFieldsArray = array(
            'idp_entity_id'  => $newIdp,
        );

        IDP_Utilities::updateDatabaseQuery($nameOfDatabase, $updateFieldsArray);
		$this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=idp',JText::_('COM_JOOMLAIDP_MSG_14'));
	}
	
    function requestForDemoPlan()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if ((!isset($post['email'])) || (!isset($post['plan'])) || (!isset($post['description']))) {
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp');
            return;
        }
        $email = $post['email'];
        $plan = $post['plan'];
        $description = trim($post['description']);
        $demo = $post['demo'];

        if (!isset($plan) || empty($description)) {
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=overview', JText::_('COM_JOOMLAIDP_MSG_1'), 'error');
            return;
        }

        $customer = new MoSamlIdpCustomer();
        $response = json_decode($customer->request_for_trial($email, $plan, $demo, $description));

        if ($response->status != 'ERROR')
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=overview', JText::_('COM_JOOMLAIDP_MSG_15').$demo.JText::_('COM_JOOMLAIDP_MSG_16'));
        else {
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=overview', JText::_('COM_JOOMLAIDP_MSG_17'), 'error');
            return;
        }

    }
	
	function saveCustomerConfigurations($email, $id, $apiKey, $token, $phone) {
        $databaseName = '#__miniorange_saml_idp_customer';
        $updateFieldsArray = array(
            'email'               => $email,
            'customer_key'        => $id,
            'api_key'             => $apiKey,
            'customer_token'      => $token,
            'admin_phone'         => $phone,
            'login_status'        => 0,
            'registration_status' => 'SUCCESS',
            'password'            => '',
            'email_count'         => 0,
            'sms_count'           => 0,
        );
        IDP_Utilities::updateDatabaseQuery($databaseName, $updateFieldsArray);
	}

    function saveAdminMail()
    {
        $post=	JFactory::getApplication()->input->post->getArray();
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $fields = array(
            $db->quoteName('email') . ' = '.$db->quote($post['admin_email']),

        );

        $conditions = array(
            $db->quoteName('id') . ' = 1'
        );

        $query->update($db->quoteName('#__miniorange_saml_idp_customer'))->set($fields)->where($conditions);
        $db->setQuery($query);
        $result = $db->execute();
        $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=sp',JText::_('COM_JOOMLAIDP_MSG_18'));
        return;
    }
	
	function contactUs(){
        $post = JFactory::getApplication()->input->post->getArray();
        if( MoSamlIdpUtility::checkEmptyOrNull( $post['mo_saml_query_email'] ) || MoSamlIdpUtility::checkEmptyOrNull( trim($post['mo_saml_query_email'])) ) {
            $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup',  JText::_('COM_JOOMLAIDP_MSG_P'), 'error');
            return;
        } else{
            $query = $post['mo_saml_query'];
            $email = $post['mo_saml_query_email'];
            $phone = $post['mo_saml_query_phone'];

            if(isset($post['mo_saml_select_plan']) && !empty($post['mo_saml_select_plan'] && $post['mo_saml_select_plan'] != 'none')
                || isset($post['number_of_users']) && !empty($post['number_of_users']))
            {
                $number_users = isset($post['number_of_users']) ? $post['number_of_users'] : '';
                if(empty($number_users)){
                    $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup',  JText::_('COM_JOOMLAIDP_MSG_Q'), 'error');
                    return;
                }
                $plan_name = $post['mo_saml_select_plan'];
                $query = "Plan Name : ".$plan_name.", Users : ".$number_users.' '.$query;
            }

            $contact_us = new MoSamlIdpCustomer();
            $submited = json_decode($contact_us->submit_contact_us($email, $phone, $query),true);
            if(json_last_error() == JSON_ERROR_NONE) {
                if(is_array($submited) && array_key_exists('status', $submited) && $submited['status'] == 'ERROR'){
                    $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup', $submited['message'],'error');
                }else{
                    if ( $submited == false ) {
                        $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=overview',  JText::_('COM_JOOMLAIDP_MSG_S'),'error');
                    } else {
                        $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=overview',  JText::_('COM_JOOMLAIDP_MSG_T'));
                    }
                }
            }
        }
    }

    function removeAccount()
    {
        $nameOfDatabase = '#__miniorange_saml_idp_customer';
        $updateFieldsArray = array(
            'email'               => '',
            'password'            => '',
            'customer_key'        => '',
            'api_key'             => '',
            'customer_token'      => '',
            'admin_phone'         => '',
            'login_status'        => 0,
            'registration_status' => 'SUCCESS',
            'email_count'         => 0,
            'sms_count'           => 0,
        );
        IDP_Utilities::updateDatabaseQuery($nameOfDatabase, $updateFieldsArray);

        $this->setRedirect('index.php?option=com_joomlaidp&view=accountsetup&tab-panel=account',  JText::_('COM_JOOMLAIDP_MSG_T'));
    }

    function importExportConfiguration()
    {
        $idp_config=IDP_Utilities::fetchDatabaseValues('#__miniorangesamlidp', 'loadAssoc');

        if (empty($idp_config['sp_entityid']) || empty($idp_config['acs_url'])) {
            $this->setRedirect('index.php?option=com_joomlaidp&tab=com_miniorange_saml&tab-panel=sp', JText::_('COM_JOOMLAIDP_MSG_U'), 'error');
            return;
        }
        $post = JFactory::getApplication()->input->post->getArray();
        require_once JPATH_SITE . DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_joomlaidp' . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'export.php';

        define("Tab_Class_Names", serialize(array(
            "sp_info" => 'mo_sp_info'
        )));

        $tab_class_name = unserialize(Tab_Class_Names);
        $configuration_array = array();
        foreach ($tab_class_name as $key => $value) {
            $configuration_array[$key] = $this->mo_get_configuration_array($value);
        }

        if ($configuration_array) {
            header("Content-Disposition: attachment; filename=miniorange-idp-config.json");
            echo(json_encode($configuration_array, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
            exit;
        }
        $this->setRedirect('index.php?option=com_joomlaidp&tab=com_miniorange_saml&tab-panel=sp', JText::_('COM_JOOMLAIDP_MSG_V'));
        return;
    }

    function mo_get_configuration_array( $class_name) {
        if($class_name=='mo_sp_info'){
            $customerResult = IDP_Utilities::fetchDatabaseValues('#__miniorangesamlidp', 'loadAssoc');
        }

		$class_object = call_user_func( $class_name . '::getConstants' );
        $mo_array = array();
        foreach ( $class_object as $key => $value ) {
            if(!empty($customerResult))
            {
                if(!empty($customerResult[$value]))
                {
                    $mo_array[$key] = $customerResult[$value];
                }
                
            }
        }
        return $mo_array;
	}
}